package com.reactnativeyappasdk;

import android.os.Bundle;
import androidx.annotation.NonNull;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.bridge.WritableMap;
import com.yappa.sdk.YappaSDK;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReadableMap;
import java.util.Map;
import java.util.HashMap;

public class SDKModule extends ReactContextBaseJavaModule {

  ReactApplicationContext context;

  SDKModule(ReactApplicationContext context) {
    super(context);
    this.context = context;
  }

  @NonNull
  @Override
  public String getName() {
    return "Yappasdk";
  }

  @ReactMethod
  public void initialize(String apiKey, String appId) {
      try{
          YappaSDK.INSTANCE.initialize(apiKey, Integer.parseInt(appId), this.context); // Lo primero que se tiene que llamar antes que nada
      }
      catch (NumberFormatException ex){
          ex.printStackTrace();
      }
  }

  @ReactMethod
  public void setContentId(String contentId) {
    YappaSDK.INSTANCE.setContentId(contentId);
  }

  @ReactMethod
  public void show() {
    YappaSDK.INSTANCE.show();
  }

  @ReactMethod
  public void close() {
    YappaSDK.INSTANCE.close();
  }

  @ReactMethod
  public void setFCMToken(String token) {
    if (!token.isEmpty()) {
      YappaSDK.INSTANCE.setFCMToken(token);
    }
  }

  @ReactMethod
  public void setContentUrl(String contentUrl) {
    YappaSDK.INSTANCE.setContentUrl(contentUrl);
  }


  @ReactMethod
  public void initializeFromNotification() {
    YappaSDK.INSTANCE.initializeFromNotification();
  }

  @ReactMethod
  public void getNotificationData(Promise promise) {
    WritableMap map = Arguments.createMap();
    Map<String, String> cachedData = YappaSDK.INSTANCE.getCachedNotificationData();

    if(cachedData != null){
      map.putString("contentUrl", cachedData.get("originalUrl"));
      map.putString("contentId", cachedData.get("contentId"));
      promise.resolve(map);
    }
    else {
      YappaSDK.INSTANCE.setCachedDataCallback((s) -> {
        map.putString("contentUrl", s.get("originalUrl"));
        map.putString("contentId", s.get("contentId"));
        System.out.println("Returned callback cache");
        promise.resolve(map);
        return null;
      });
    }
  }




  @ReactMethod
  public void handleNotification(ReadableMap remoteMessageMap) {
    ReadableMap notification = remoteMessageMap.getMap("notification");
    ReadableMap data = remoteMessageMap.getMap("data");
    Bundle mappedData = new Bundle();

    if(data != null && notification != null){
      for (String key : data.toHashMap().keySet()) {
        if(data.getType(key) == ReadableType.String){
          mappedData.putString(key, data.getString(key));
        }
      }

      if(notification.hasKey("title") && notification.hasKey("body")){
        YappaSDK.INSTANCE.handleNotification(mappedData, (s, s2) -> {
          return null;
        });
      }
    }
  }

  @ReactMethod
  public void handleNotificationForeground(ReadableMap remoteMessageMap) {
    ReadableMap notification = remoteMessageMap.getMap("notification");
    ReadableMap data = remoteMessageMap.getMap("data");
    HashMap<String, String> mappedData = new HashMap<>();

    if(data != null && notification != null){
      for (String key : data.toHashMap().keySet()) {
        if(data.getType(key) == ReadableType.String){
            mappedData.put(key, data.getString(key));
        }
      }

      if(notification.hasKey("title") && notification.hasKey("body")){
        YappaSDK.INSTANCE.displayNotification(mappedData, notification.getString("title"), notification.getString("body"));
      }
    }
  }
}
